"""
Module FACT_data_project

The FACTDatasetIO module serves as a comprehensive toolkit for managing, processing, and exporting datasets within the FACT framework. It integrates functionality for executing data processing workflows, retrieving results from various data sources, and exporting data to multiple formats. This module acts as a bridge between high-level workflow definitions and the detailed, task-specific operations needed to handle data effectively in the FACT ecosystem.

## Dependencies
- FACTWorkflowManager: Utilized for orchestrating and executing defined workflows.
- FACTResultsIO: Handles the input/output operations related to data retrieval and storage.
- External Libraries: Uses TOML for configuration parsing, DataFrames for data manipulation, JSON, CSV, StatFiles, Parquet, and SQLite for data serialization and storage.

## Components
- WorkflowRunner.jl: Implements the logic for running and managing data processing workflows.
- DataFrameManager.jl: Provides utilities for manipulating DataFrames, including transformations and aggregations.
- ResultsReader.jl: Contains functionality for reading and interpreting results from data processing tasks.
- FileWriter.jl: Offers methods for writing data to files in various formats, facilitating data export and sharing.

## Functionality
- **runWorkflow**: Executes a data processing workflow defined in a TOML file, orchestrating the various tasks according to their dependencies and the specified execution order.
- **get_joined_results_dataset**: Retrieves and joins results from multiple data processing tasks, forming a comprehensive dataset ready for analysis or export.

## Usage
This module is designed to be used as part of larger data processing and analysis projects within the FACT framework. It can be invoked programmatically from Julia scripts or applications that require robust data handling capabilities.

Example:
To execute a workflow and retrieve the results, you would use the runWorkflow function from the FACTDatasetIO module with the path to your workflow TOML file as the argument. Similarly, to get a comprehensive dataset by joining results from multiple tasks, use the get_joined_results_dataset function, specifying the necessary arguments as documented.

This module encapsulates the core functionalities needed for effective data management in the FACT framework, streamlining the process from workflow execution to data export.
"""
module FACT_data_project

using FACT_workflow_manager, OrchestratorRegistry
using TOML, DataFrames, JSON, CSV, ReadStatTables, Parquet, SQLite

include("utils/ResultsReader.jl")
include("utils/FileWriter.jl")
include("utils/DataFrameManager.jl")
export get_joined_results_dataset
include("cmd/WorkflowRunner.jl")
export runWorkflow
include("cmd/DownloadDataset.jl")
export download_dataset

end