"""
    download_dataset(project_filename::String, output_filename::String)

Downloads and processes a dataset defined in a project file and saves the resulting dataset to an output file in the specified format.

Arguments:
- `project_filename::String`: The name of the project file (with extension) located in the `projects` directory. This file defines the dataset to be downloaded.
- `output_filename::String`: The name of the output file (with extension) where the processed dataset will be saved. This file will be created in the `data` directory.

Workflow:
1. Constructs full paths for the input project file and the output file.
2. Logs into the registry and retrieves an authentication token.
3. Starts a results container and establishes a connection to retrieve data.
4. Processes the dataset defined in the project file using the connection.
5. Writes the processed dataset to the specified output file in the desired format.
6. Stops the results container after the operation is complete.

Supported Output Formats:
- `.json`: Saves the dataset as a JSON file.
- `.csv`: Saves the dataset as a CSV file.
- `.dta`: Saves the dataset in Stata format.
- `.parquet`: Saves the dataset as a Parquet file.
- `.sqlite`: Saves the dataset to an SQLite database.

Example Usage:
This example downloads a dataset defined in `example_project.toml` and saves it as `output.parquet`:
> download_dataset("example_project.toml", "output.parquet")
"""
function download_dataset(project_filename::String, output_filename::String)
    filename = joinpath("./projects", project_filename)
    outputFilename = joinpath("./data", output_filename)

    encoded_auth = registry_login("registry.nilu.no", "FACT_token", "glpat-1zG-TQx___xLsPjj2vsG")
    connection_info = get_results_container(encoded_auth)

    try
        result = get_joined_results_dataset(filename, connection_info)            
        write_results_dataset(result, outputFilename)
    catch e
        println("An error occurred downloading dataset: ", e)
    end

    stop_container("fact_results", auth_token=encoded_auth)
end
