"""
    get_joined_results_dataset(filename::String, connection_info::Dict) -> Dict{String, DataFrame}

Reads task and output configurations from a TOML file specified by `filename` and retrieves the results for each task, joining them as specified in the output configuration. The function constructs a dictionary mapping each task or join name to its respective DataFrame. Single tasks are added directly, while joins are processed according to the join configurations defined in the TOML file.

# Arguments
- `filename::String`: The path to the TOML file containing the workflow definition, including tasks and output configurations.
- `connection_info::Dict`: A dictionary containing the database connection information needed to retrieve task results.

# Returns
A dictionary where keys are task or join names and values are DataFrames representing the results of individual tasks or the results of specified joins among tasks.

# Workflow
- Parses the TOML file to extract tasks and output configurations.
- Retrieves results for each task defined in the tasks section, using the provided `connection_info`.
- Processes joins as defined in the output section, creating a new DataFrame for each join and adding it to the output dictionary with the specified join name.
- Adds single task results directly to the output dictionary using the task name as the key.

# Example
Assuming `project.toml` defines tasks and their dependencies, and `connection_info` provides the necessary database connection details:

> requested_df = get_joined_results_dataset("project.toml", connection_info)

This will return a dictionary containing DataFrames for each task and join specified in `project.toml`, ready for further analysis or export.
"""
function get_joined_results_dataset(filename::String, connection_info::Dict)
    # Read [output] and [tasks] sections from project.TOML
    tasks, output = parseOutputflow(filename)

    # Initialize empty dictionary for task results
    results = Dict()

    # Iterate over tasks, capturing both task names and their details
    for (task_name, task_details) in tasks
        df = getResultsFromTask(task_details, connection_info, tasks)
        results[task_name] = df
    end

    # Initialize empty dictionary for the final output
    requested_df = Dict{String, DataFrame}()

    # Process joins if 'join' exists
    if haskey(output, "join")
        for (join_name, join_details) in output["join"]
            join_type_str = get(join_details, "type", "")
            join_type = isempty(join_type_str) ? :outer : Symbol(join_type_str)
        
            dfnames = join_details["dataframes"]
            df = results[dfnames[1]]

            for i in 2:length(dfnames)
                df = join_on_matching_columns(df, results[dfnames[i]], join_type=join_type)
            end
        
            requested_df[join_name] = df
        end
    end

    # Process singles if 'single' exists
    if haskey(output, "single") && haskey(output["single"], "dataframes")
        singlenames = output["single"]["dataframes"]
        for single in singlenames
            df = results[single]
            requested_df[single] = df
        end
    end

    # Return the dictionary of joined and independent tables
    return requested_df
end

"""
    join_on_matching_columns(df_left::DataFrame, df_right::DataFrame; join_type::Symbol=:outer) -> DataFrame

Performs a join operation between two DataFrames (`df_left` and `df_right`) based on columns with matching names. The type of join performed can be specified using the `join_type` parameter, which defaults to an outer join if not provided.

# Arguments
- `df_left::DataFrame`: The left DataFrame in the join operation.
- `df_right::DataFrame`: The right DataFrame in the join operation.
- `join_type::Symbol`: (Optional) The type of join to perform, specified as a Symbol. Accepted values include `:inner`, `:left`, `:right`, `:outer`, `:semi`, and `:anti`, mirroring the join types supported by the DataFrames.jl package. Defaults to `:outer`.

# Returns
`DataFrame`: The result of joining `df_left` and `df_right` based on columns with matching names, according to the specified `join_type`.

# Example
Given two DataFrames `df1` and `df2` with some columns having matching names:

> joined_df = join_on_matching_columns(df1, df2, join_type=:inner)

This will return a new DataFrame resulting from an inner join of `df1` and `df2` on all columns that exist in both DataFrames.
"""
function join_on_matching_columns(df_left::DataFrame, df_right::DataFrame; join_type::Symbol=:outer)
    # Identify the matching column names between the two dataframes
    matching_columns = intersect(names(df_left), names(df_right))
    
    # Dispatch the join operation based on join_type
    joined_df = nothing
    if join_type == :inner
        joined_df = innerjoin(df_left, df_right, on = matching_columns)
    elseif join_type == :left
        joined_df = leftjoin(df_left, df_right, on = matching_columns)
    elseif join_type == :right
        joined_df = rightjoin(df_left, df_right, on = matching_columns)
    elseif join_type == :outer
        joined_df = outerjoin(df_left, df_right, on = matching_columns)
    elseif join_type == :semi
        joined_df = semijoin(df_left, df_right, on = matching_columns)
    elseif join_type == :anti
        joined_df = antijoin(df_left, df_right, on = matching_columns)
    elseif join_type == :cross
        # Note: crossjoin does not use 'on' parameter
        joined_df = crossjoin(df_left, df_right)
    else
        throw(ArgumentError("Unsupported join type: $join_type"))
    end
    
    return joined_df
end
