"""
    write_results_dataset(result::Dict, filename::String)

Writes the contents of the `result` dictionary, which maps names to DataFrames, to disk in various formats based on the file extension specified in `filename`. The function supports exporting data to JSON, CSV, Stata, Parquet, and SQLite formats, creating one file per DataFrame in the dictionary, except for SQLite where all DataFrames are written to a single database file.

# Arguments
- `result::Dict`: A dictionary mapping unique names (as keys) to DataFrames (as values).
- `filename::String`: The base name for output files, including the desired extension which determines the format of the output. The actual filenames will be constructed by appending the key from `result` to the base name.

# Supported Formats
- `.json`: Exports each DataFrame as a separate JSON file.
- `.csv`: Exports each DataFrame as a separate CSV file.
- `.dta`: Exports each DataFrame as a separate Stata file.
- `.parquet`: Exports each DataFrame as a separate Parquet file.
- `.sqlite`: Writes all DataFrames to a single SQLite database file, with each DataFrame in its own table named after its key in `result`.

# Errors
- Raises an error if the file extension is not supported, indicating the format is unrecognized.

# Example
Given a result dictionary `resultDict` with DataFrames and a filename 'output.json':

> write_results_dataset(resultDict, "output.json")

This will export each DataFrame in `resultDict` to a separate JSON file, named with the corresponding key in `resultDict` and prefixed with 'output_'.
"""
function write_results_dataset(result::Dict, filename::String)

    # Define a dictionary to map format arguments to file extensions
    extension_map = Dict(
        ".json" => "json",
        ".csv" => "csv",
        ".dta" => "stata",
        ".parquet" => "parquet",
        ".sqlite" => "sqlite"
    )

    # Split the filename into the name and extension parts
    basename, extension = splitext(filename)

    if !haskey(extension_map, extension)
        error("Unsupported format: $extension")
    else 
        format = extension_map[extension]
        basename, extension = splitext(filename)
        #println("Writing $format file: $filename")
        try
            if format == "json"
                for (key, df) in result
                    # Construct the output filename for each DataFrame
                    output_filename = basename * "_" * key * extension
                    json_string = JSON.json(Array(df))
                    # Write the JSON string to the file
                    open(output_filename, "w") do io
                        write(io, json_string)
                    end
                end
            elseif format == "csv"
                for (key, df) in result
                    # Construct the output filename for each DataFrame
                    output_filename = basename * "_" * key * extension
                    CSV.write(output_filename, df)
                end
            elseif format == "stata"
                for (key, df) in result
                    # Construct the output filename for each DataFrame
                    output_filename = basename * "_" * key * extension
                    preprocess_dataframe_to_numerical!(df)
                    ReadStatTables.writestat(output_filename, df)
                end
            elseif format == "parquet"
                for (key, df) in result
                    # Construct the output filename for each DataFrame
                    output_filename = basename * "_" * key * extension
                    preprocess_dataframe_to_numerical!(df)
                    Parquet.write_parquet(output_filename, df)
                end
            elseif format == "sqlite"
                db = SQLite.DB(filename)
                for (key, df) in result
                    preprocess_dataframe_to_numerical!(df)
                    SQLite.load!(df, db, key)
                end
                SQLite.close(db)
            else
                error("Unsupported format: $format")
            end
        catch e
            println("An error occurred while writing output files: $e")
        end
    end
end

"""
    preprocess_dataframe_to_numerical!(df::DataFrame)

Transforms columns of a DataFrame to numerical values, specifically to `Float64`, except for columns designated to remain as strings. This function is particularly useful for preparing dataset columns for statistical analysis or machine learning models that require numerical input. It attempts to parse each string in the DataFrame to a `Float64` and replaces non-convertible strings with `NaN` to maintain numerical operations' integrity.

## Arguments
- `df::DataFrame`: The DataFrame to be processed. This function modifies the DataFrame in place.

## Behavior
- Columns named `"geo_id"` are skipped to preserve any geographic identifiers or other string-based identifiers that should not be converted to numbers.
- For other columns, the function attempts to convert string representations of numbers into `Float64`. If a value cannot be parsed as a `Float64`, it is replaced with `NaN` (Not a Number), a standard IEEE floating-point representation for undefined or unrepresentable values.

## Example
Given a DataFrame `df` with mixed string numerical values and identifiers:

- `df = DataFrame(geo_id = ["id1", "id2"], measurement = ["1.23", "4.56"], category = ["A", "B"])`

Applying `preprocess_dataframe_to_numerical!(df)` would result in `df` having its `measurement` column converted to `Float64`, with `category` values replaced by `NaN`, and `geo_id` left unchanged.

## Notes
- This preprocessing removes the "shape_obj" column from the DataFrame if present.
- It's important to backup your DataFrame before applying this function if you need to preserve the original data.
- Handling of `NaN` values should be considered in subsequent data processing or analysis stages.
"""
function preprocess_dataframe_and_remove_shape_obj!(df::DataFrame)
    # Remove the shape_obj column from the DataFrame
    if "shape_obj" in names(df)
        select!(df, Not(:shape_obj))
    end

    for col_name in names(df)
        # Skip conversion if column is meant to be string
        if col_name in ["geo_id", "o_geo_id", "d_geo_id", "industry"]
            continue
        end

        # Attempt conversion to Float64, replace non-convertible strings with NaN
        df[!, col_name] = map(x -> try
                                        parse(Float64, x)
                                  catch
                                        NaN # Use NaN for non-convertible values
                                  end, 
                             df[!, col_name])
    end
end
