"""
    getResultsFromTask(task_definition::Dict, connection_info::Dict) -> DataFrame

Retrieves the results of a specific task from a database, based on the task's definition and connection information. 
This function establishes a database connection, queries the database for the task's results using details specified 
in `task_definition`, and then closes the database connection.

# Arguments
- `task_definition::Dict`: A dictionary containing the task's definition, including the function name, arguments, 
  and parameters dictionary to query the database.
- `connection_info::Dict`: A dictionary containing the database connection information required to establish the connection.

# Returns
`DataFrame`: A DataFrame containing the results of the database query based on the task's definition. Returns an 
empty DataFrame if no results are found or if any required information is missing in `task_definition`.

# Example
To retrieve results for a task with specific function name, arguments, and parameters, and given database connection info:

> task_definition = Dict("function" => "computeStats", "arguments" => "arg1", "parameters_dict" => Dict("param1" => "value1"))
> connection_info = Dict("host" => "localhost", "db" => "mydatabase")
> result = getResultsFromTask(task_definition, connection_info)

This example queries the database for the results of the `computeStats` function with specified arguments and parameters,
returning the results as a DataFrame.
"""
function getResultsFromTask(task_definition::Dict, connection_info::Dict, tasks::Dict)
    result = DataFrame()
  
    # Setup connection
    results_connection = get_connection(connection_info)

    # Extract the required information
    functionName = String(get(task_definition, "function", ""))  # Default to empty string if not found
    arguments = String(get(task_definition, "arguments", ""))  # Adjust based on actual key if different
    parametersDict = Dict(get(task_definition, "parameters_dict", Dict()))
    dependencies_info = Dict{String, Dict}()
    for dep in task_definition["dependencies"]
        dependencies_info[dep] = tasks[dep]
    end
    processed_parameters = process_task_parameters(parametersDict, dependencies_info, results_connection)

    # get the stored result
    result = get_table(results_connection, functionName, arguments, processed_parameters)
    
    # Drop connection
    close_connection(results_connection)

    return result
end

"""
    parseOutputflow(filePath::String) -> (Dict, Dict)

Parses a workflow file specified by `filePath`, extracting configurations for tasks and output definitions. 
This function utilizes the TOML format for the workflow file, providing a structured way to define and 
retrieve task and output configurations.

# Arguments
- `filePath::String`: The path to the workflow definition file in TOML format.

# Returns
- `(Dict, Dict)`: A tuple containing two dictionaries. The first dictionary contains the tasks configuration, 
  and the second dictionary contains the output configuration as defined in the workflow file.

# Example
To parse a workflow definition from 'workflow.toml' and retrieve task and output configurations:

> tasks, output = parseOutputflow("workflow.toml")

This will return the tasks and output configurations as two separate dictionaries, based on the content of 'workflow.toml'.
"""
function parseOutputflow(filePath::String)
    # Parse the workflow file
    config = TOML.parsefile(filePath)
    tasks = config["tasks"]
    output = config["output"]
    return tasks, output
end